@@ -8,5 +8,11 @@ from lightwood.analysis.nc.calibrate import ICP
 from lightwood.analysis.helpers.acc_stats import AccStats
 from lightwood.analysis.helpers.feature_importance import GlobalFeatureImportance
 
+try:
+    from lightwood.analysis.helpers.shap import ShapleyValues
+except Exception:
+    ShapleyValues = None
 
-__all__ = ['model_analyzer', 'explain', 'ICP', 'AccStats', 'GlobalFeatureImportance', 'BaseAnalysisBlock']
+
+__all__ = ['model_analyzer', 'explain', 'ICP', 'AccStats',
+           'GlobalFeatureImportance', 'BaseAnalysisBlock', 'ShapleyValues']
