@@ -614,9 +614,7 @@ def _add_implicit_values(json_ai: JsonAI) -> JsonAI:
                 "target_name": "$target",
                 "target_dtype": "$dtype_dict[self.target]",
                 "explainer_blocks": "$analysis_blocks",
-                "fixed_confidence": "$pred_args.fixed_confidence",
-                "anomaly_error_rate": "$pred_args.anomaly_error_rate",
-                "anomaly_cooldown": "$pred_args.anomaly_cooldown",
+                "pred_args": "$pred_args",
             },
         },
         "analysis_blocks": [
@@ -632,7 +630,7 @@ def _add_implicit_values(json_ai: JsonAI) -> JsonAI:
                 "module": "AccStats",
                 "args": {"deps": ["ICP"]},
             },
-        ],
+        ] if problem_definition.use_default_analysis else [],
         "timeseries_transformer": {
             "module": "transform_timeseries",
             "args": {
