@@ -1,3 +1,4 @@
+import numpy as np
 import pandas as pd
 from lightwood.api.types import TimeseriesSettings
 
@@ -29,18 +30,21 @@ def get_inferred_timestamps(df: pd.DataFrame, col: str, deltas: dict, tss: Times
 def add_tn_conf_bounds(data: pd.DataFrame, tss_args: TimeseriesSettings):
     """
     Add confidence (and bounds if applicable) to t+n predictions, for n>1
-    @TODO: active research question: how to guarantee 1-e coverage for t+n, n>1
-    for now, we replicate the width and conf obtained for t+1
+    TODO: active research question: how to guarantee 1-e coverage for t+n, n>1
+    For now, (conservatively) increases width by the confidence times the log of the time step (and a scaling factor).
     """
     for col in ['confidence', 'lower', 'upper']:
         data[col] = data[col].astype(object)
 
     for idx, row in data.iterrows():
+        error_increase = [row['confidence']] + \
+                         [row['confidence'] * np.log(np.e + t / 2)  # offset by e so that y intercept is 1
+                          for t in range(1, tss_args.nr_predictions)]
         data['confidence'].iloc[idx] = [row['confidence'] for _ in range(tss_args.nr_predictions)]
 
         preds = row['prediction']
         width = row['upper'] - row['lower']
-        data['lower'].iloc[idx] = [pred - width / 2 for pred in preds]
-        data['upper'].iloc[idx] = [pred + width / 2 for pred in preds]
+        data['lower'].iloc[idx] = [pred - (width / 2) * modifier for pred, modifier in zip(preds, error_increase)]
+        data['upper'].iloc[idx] = [pred + (width / 2) * modifier for pred, modifier in zip(preds, error_increase)]
 
     return data
